﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataPreprocessing {
  [Item("DoublePreprocessingDataColumn", "")]
  [StorableClass]
  public sealed class DoublePreprocessingDataColumn : PreprocessingDataColumn<double> {

    #region Constructor, Cloning & Persistence

    public DoublePreprocessingDataColumn()
      : base() { }

    public DoublePreprocessingDataColumn(string name, IEnumerable<double> values)
      : base(name, values) { }

    private DoublePreprocessingDataColumn(DoublePreprocessingDataColumn original, Cloner cloner)
      : base(original, cloner) { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new DoublePreprocessingDataColumn(this, cloner);
    }

    [StorableConstructor]
    private DoublePreprocessingDataColumn(bool deserializing)
      : base(deserializing) { }

    #endregion

    public override bool IsValidValue(double value) {
      return !(double.IsNaN(value) || double.IsInfinity(value));
    }

    protected override double DefaultValue { get { return double.NaN; } }

    #region Statistics
    public double GetRange(IEnumerable<int> indices = null) { return GetMax(indices) - GetMin(indices); }
    public double GetMean(IEnumerable<int> indices = null) { return GetValidValues(indices).DefaultIfEmpty(DefaultValue).Average(); }
    public override double GetMedian(IEnumerable<int> indices = null) { return GetValidValues(indices).DefaultIfEmpty(DefaultValue).Median(); } // IEnumerable<doube> version is faster
    public double GetStandardDeviation(IEnumerable<int> indices = null) { return GetValidValues(indices).DefaultIfEmpty(DefaultValue).StandardDeviation(); }
    public double GetVariance(IEnumerable<int> indices = null) { return GetValidValues(indices).DefaultIfEmpty(DefaultValue).Variance(); }
    public override double GetQuantile(double alpha, IEnumerable<int> indices = null) { return GetValidValues(indices).DefaultIfEmpty(double.NaN).Quantile(alpha); } // IEnumerable<doube> version is faster
    #endregion

    #region IStringConvertibleColumn
    public override bool Validate(string value, out string errorMessage) {
      double val;
      bool valid = double.TryParse(value, out val) || string.IsNullOrEmpty(value);
      errorMessage = string.Empty;
      if (!valid) {
        var sb = new StringBuilder();
        sb.Append("Invalid Value (Valid Value Format: \"");
        sb.Append(FormatPatterns.GetDoubleFormatPattern());
        sb.Append("\")");
        errorMessage = sb.ToString();
      }
      return valid;
    }
    public override string GetValue(int index) {
      return Values[index].ToString("r");
    }
    public override bool SetValue(string value, int index) {
      double val;
      if (double.TryParse(value, out val)) {
        Values[index] = val;
        return true;
      } else if (string.IsNullOrEmpty(value)) {
        Values[index] = double.NaN;
        return true;
      } else {
        return false;
      }
    }
    #endregion
  }
}